#include "fir.h"

CFIR::CFIR( int * hArray, int tapOfFilter )
{
	int i;
	
	h = hArray;
	taps = tapOfFilter;
	delay = new int[ taps ];
	delayptr = delay;
	
	for ( i=0; i<taps; i++ )
		delay[i] = 0;
}

CFIR::~CFIR( void )
{
	delete delay;
}

int CFIR::filter( int x )
{
		// i0,l0,b0 : delay ptr
		// i6 : end address of h
		// cntr : taps-1
		// input : mx0
        // output: mr1
	asm( 
		"dis m_mode;\n"
		"i0=%2;\n"			// delay ptr
		"m1=1;\n"
		"reg(b0)=%3;\n"		// base address of buffer
		"i6=%4;\n"			// end address of h
		"mx0=%5;\n"			// input x
		"my0=%6;\n"			// taps 
    	"l0=my0;\n"			// 
    	"ar=my0-1;\n"		// taps -1
    	"cntr=ar;\n"
    	
		
    	"mr=0;\n"                       			// SUM=0
    	"dm(i0+=m1)=mx0;\n"             			// ŐṼTvXgA
    	"mx0=dm(i0+=m1),my0=pm(i6+=m5);\n"	    	// v[h

    	"do fir_loop until ce;\n"
	"fir_loop:\n"
        "mr=mr+mx0*my0(ss), mx0=dm(i0+=m1),my0=pm(i6+=m5);\n" // Ϙa
    	"mr=mr+mx0*my0(rnd);\n"                               // Ō̐Ϙa    rts;        
    	"%0=mr1;\n"
    	"%1=i0;\n"								// i0 : delay ptr
    	"ena m_mode;\n"
    	"l0=0;\n"
	: "=e"( x ), "=e"( delayptr )
	: "e"( delayptr ), "e"( delay ), "e"( &h[taps-1] ), "e"( x ), "e"( taps )
	: "memory", "mr0","mr1","mr2","mx0","i0","m1","i6","m5","cntr","my0", "ar", "l0"
	);
	
	return( x );
}

void CFIR::block_filter( 	short * src, int src_step, 
							short * dst, int dst_step,
							int count )
{
	int i, j, k;
	
	for( i=j=k=0; k<count; i+=src_step, j+=dst_step, k++ )
	
		// i0,l0,b0 : delay ptr
		// i6 : end address of h
		// cntr : taps-1
		// input : mx0
        // output: mr1
	asm( 
		"dis m_mode;\n"
		"i0=%2;\n"			// delay ptr
		"m1=1;\n"
		"reg(b0)=%3;\n"		// base address of buffer
		"i6=%4;\n"			// end address of h
		"mx0=%5;\n"			// input x
		"my0=%6;\n"			// taps 
    	"l0=my0;\n"			// 
    	"ar=my0-1;\n"		// taps -1
    	"cntr=ar;\n"
    	
		
    	"mr=0;\n"                       			// SUM=0
    	"dm(i0+=m1)=mx0;\n"             			// ŐṼTvXgA
    	"mx0=dm(i0+=m1),my0=pm(i6+=m5);\n"	    	// v[h

    	"do fir_loop_block until ce;"
	"fir_loop_block:"
        "mr=mr+mx0*my0(ss), mx0=dm(i0+=m1),my0=pm(i6+=m5);" // Ϙa
    	"mr=mr+mx0*my0(rnd);"                               // Ō̐Ϙa    rts;        
    	"%0=mr1;"
    	"%1=i0;"								// i0 : delay ptr
    	"ena m_mode;"
    	"l0=0;\n"

	: "=e"( dst[j] ), "=e"( delayptr )
	: "e"( delayptr ), "e"( delay ), "e"( &h[taps-1] ), "e"( src[i] ), "e"( taps )
	: "memory", "mr0","mr1","mr2","mx0","i0","m1","i6","m5","cntr","my0", "ar", "l0"
	);
	
}



void CFIR::block_filter2( 	short * src, int src_step, 
							short * dst, int dst_step,
							int count )
{
	
		// i0,l0,b0 : delay ptr
		// i6,l6,b6 : h ( i6 point end of h )
		// ar : taps -1;
		// i2, m2 : src
		// i3, m3 : dst
		
		
		// input : mx0
        // output: mr1
	asm( 
		"dis m_mode;\n"
		"i0=%1;\n"			// delay ptr
		"m1=1;\n"
		"reg(b0)=%2;\n"		// base address of buffer
		"reg(b6)=%3;\n"		// address of h
		"my0=reg(b6);\n"
		"i6=my0;\n"			//
		"my0=%4;\n"			// taps 
    	"l0=my0;\n"			// 
    	"l6=my0;\n"
    	"ar=my0-1;\n"		// taps -1
		"modify(i6,m5);\n"	// i6 : end of h
    	
    	"i2=%5;\n"			// src
    	"m2=%6;\n"			// src_step
    	"i3=%7;\n"			// dst
    	"m3=%8;\n"			// dst_step
    	"cntr=%9;\n"		// count;
    	    	
		"do fir_loop_block_outer until ce;\n"

		"cntr=ar;\n"    	// taps -1
		"mx0=dm(i2+=m2);\n"
    	"mr=0;\n"                       			// SUM=0
    	"dm(i0+=m1)=mx0;\n"             			// ŐṼTvXgA
    	"mx0=dm(i0+=m1),my0=pm(i6+=m5);\n"	    	// v[h

    	"do fir_loop_block2 until ce;"
	"fir_loop_block2:"
        "mr=mr+mx0*my0(ss), mx0=dm(i0+=m1),my0=pm(i6+=m5);" // Ϙa
    	"mr=mr+mx0*my0(rnd);"                               // Ō̐Ϙa    rts;        
    	
    "fir_loop_block_outer:\n"
    	"dm(i3+=m3)=mr1;\n"
    	
    	"%0=i0;\n"								// i0 : delay ptr
    	"ena m_mode;\n"
    	"l0=0;\n"
    	"l6=0;\n"
    	
	: "=e"( delayptr )
	: "e"( delayptr ), "e"( delay ), "e"( h ), "e"( taps ),
	  "e"( src ), "e"( src_step ), "e"( dst ), "e"( dst_step ), "e"( count )
	: "memory", "mr0","mr1","mr2","mx0","i0","m1","i6","m5","cntr","my0", "ar",
	  "i2", "m2", "i2", "i3", "l0", "b0", "m6", "l6", "b6"
	);
	
}
